#
#                           TERMS AND CONDITIONS
#                                   FOR
#                         OPEN SOURCE CODE LICENSE
#                               Version 1.1
# 
# Japan Registry Services Co., Ltd. ("JPRS"), a Japanese corporation
# having its head office at Chiyoda First Bldg. East 13F 3-8-1 Nishi-Kanda,
# Chiyoda-ku, Tokyo 101-0065, Japan, grants you the license for open source
# code specified in EXHIBIT A the "Code" subject to the following Terms and
# Conditions ("OSCL").
# 
# 1. License Grant.
#   JPRS hereby grants you a worldwide, royalty-free, non-exclusive
#   license, subject to third party intellectual property claims:
#   (a) under intellectual property rights (other than patent or
#       trademark) licensable by JPRS to use, reproduce, modify, display,
#       perform, sublicense and distribute the Code (or portions thereof)
#       with or without modifications, and/or as part of a derivative work;
#       or
#   (b) under claims of the infringement through the making, using,
#       offering to sell and/or otherwise disposing the JPRS Revised Code
#       (or portions thereof);
#   (c) the licenses granted in this Section 1(a) and (b) are effective on
#       the date JPRS first distributes the Code to you under the terms of
#       this OSCL;
#   (d) Notwithstanding the above stated terms, no patent license is
#       granted:
#       1)  for a code that you delete from the Code;
#       2)  separate from the Code; or
#       3)  for infringements caused by:
#            i) modification of the Code; or
#           ii) combination of the Code with other software or devices.
# 
# 2. Consents.
#   You agree that:
#   (a) you must include a copy of this OSCL and the notice set forth in
#       EXHIBIT A with every copy of the Code you distribute;
#   (b) you must include a copy of this OSCL and the notice set forth in
#       EXHIBIT A with every copy of binary form of the Code in the
#       documentation and/or other materials provided with the distribution;
#   (c) you may not offer or impose any terms on any source code version
#       that alters or restricts the applicable version of this OSCL or
#       the recipients' rights hereunder.
#   (d) If the terms and conditions are set forth in EXHIBIT A, you must
#       comply with those terms and conditions.
# 
# 3. Proprietary Information.
#   All trademarks, service marks, patents, copyrights, trade secrets, and
#   other proprietary rights in or related to the Code are and will remain
#   the exclusive property of JPRS or its licensors, whether or not
#   specifically recognized or perfected under local law except specified
#   in this OSCL; provided however you agree and understand that the JPRS
#   name may not be used to endorse or promote this Code without prior
#   written approval of JPRS.
# 
# 4. WARRANTY DISCLAIMER.
#   JPRS MAKES NO REPRESENTATIONS AND WARRANTIES REGARDING THE USE OF THE
#   CODE, NOR DOES JPRS MAKE ANY REPRESENTATIONS THAT THE CODE WILL BECOME
#   COMMERCIALLY AVAILABLE. JPRS, ITS AFFILIATES, AND ITS SUPPLIERS DO NOT
#   WARRANT OR REPRESENT THAT THE CODE IS FREE OF ERRORS OR THAT THE CODE
#   IS SUITABLE FOR TRANSLATION AND/OR LOCALIZATION. THE CODE IS PROVIDED
#   ON AN "AS IS" BASIS AND JPRS AND ITS SUPPLIERS HAVE NO OBLIGATION TO
#   CORRECT ERRORS OR TO SUPPORT THE CODE UNDER THIS OSCL FOR ANY REASON.
#   TO THE FULL EXTENT PERMITTED BY LAW, ALL OBLIGATIONS ARE HEREBY
#   EXCLUDED WHETHER EXPRESS, STATUTORY OR IMPLIED UNDER LAW, COURSE OF
#   DEALING, CUSTOM, TRADE USAGE, ORAL OR WRITTEN STATEMENT OR OTHERWISE,
#   INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY
#   OR FITNESS FOR A PARTICULAR PURPOSE CONCERNING THE CODE.
# 
# 5. NO LIABILITY.
#   UNDER NO CIRCUMSTANCES SHALL JPRS AND/OR ITS AFFILIATES, LICENSORS, OR
#   REPRESENTATIVES BE LIABLE FOR ANY DAMAGES INCLUDING BUT NOT LIMITED TO
#   CONSEQUENTIAL, INDIRECT, SPECIAL, PUNITIVE OR INCIDENTAL DAMAGES,
#   WHETHER FORESEEABLE OR UNFORESEEABLE, BASED ON YOUR CLAIMS, INCLUDING,
#   BUT NOT LIMITED TO, CLAIMS FOR LOSS OF DATA, GOODWILL, PROFITS, USE OF
#   MONEY, INTERRUPTION IN USE OR AVAILABILITY OF DATA, STOPPAGE, IMPLIED
#   WARRANTY, BREACH OF CONTRACT, MISREPRESENTATION, NEGLIGENCE, STRICT
#   LIABILITY IN TORT, OR OTHERWISE.
# 
# 6. Indemnification.
#   You hereby agree to indemnify, defend, and hold harmless JPRS for any
#   liability incurred by JRPS due to your terms of warranty, support,
#   indemnity, or liability offered by you to any third party.
# 
# 7. Termination.
# 7.1 This OSCL shall be automatically terminated in the events that:
#   (a) You fail to comply with the terms herein and fail to cure such
#       breach within 30 days of becoming aware of the breach;
#   (b) You initiate patent or copyright infringement litigation against
#       any party (including a cross-claim or counterclaim in a lawsuit)
#       alleging that the Code constitutes a direct or indirect patent or
#       copyright infringement, in such case, this OSCL to you shall
#       terminate as of the date such litigation is filed;
# 7.2 In the event of termination under Sections 7.1(a) or 7.1(b) above,
#     all end user license agreements (excluding distributors and
#     resellers) which have been validly granted by You or any distributor
#     hereunder prior to termination shall survive termination.
#
# 
# 8. General.
#   This OSCL shall be governed by, and construed and enforced in
#   accordance with, the laws of Japan. Any litigation or arbitration
#   between the parties shall be conducted exclusively in Tokyo, Japan
#   except written consent of JPRS provides other venue.
# 
# 
#                                EXHIBIT A
# 
# The original open source code of idnkit-2 is idnkit-1.0 developed and
# conceived by Japan Network Information Center ("JPNIC"), a Japanese
# association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
# Chiyoda-ku, Tokyo 101-0047, Japan, and JPRS modifies above original code
# under following Terms and Conditions set forth by JPNIC.
# 
#                                  JPNIC
# 
# Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved.
# 
# By using this file, you agree to the terms and conditions set forth bellow.
# 
#                       LICENSE TERMS AND CONDITIONS
# 
# The following License Terms and Conditions apply, unless a different
# license is obtained from Japan Network Information Center ("JPNIC"),
# a Japanese association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
# Chiyoda-ku, Tokyo 101-0047, Japan.
# 
# 1. Use, Modification and Redistribution (including distribution of any
#    modified or derived work) in source and/or binary forms is permitted
#    under this License Terms and Conditions.
# 
# 2. Redistribution of source code must retain the copyright notices as they
#    appear in each source code file, this License Terms and Conditions.
# 
# 3. Redistribution in binary form must reproduce the Copyright Notice,
#    this License Terms and Conditions, in the documentation and/or other
#    materials provided with the distribution. For the purposes of binary
#    distribution the "Copyright Notice" refers to the following language:
#    "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
# 
# 4. The name of JPNIC may not be used to endorse or promote products
#    derived from this Software without specific prior written approval of
#    JPNIC.
# 
# 5. Disclaimer/Limitation of Liability: THIS SOFTWARE IS PROVIDED BY JPNIC
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JPNIC BE LIABLE
#    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
#    BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
#    ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
# 
# 
#                        JPRS Public License Notice
#                                   For
#                                idnkit-2.
# 
# The contents of this file are subject to the Terms and Conditions for
# the Open Source Code License (the "OSCL"). You may not use this file
# except in compliance with above terms and conditions. A copy of the OSCL
# is available at <http://jprs.co.jp/idn/>.
# The JPRS Revised Code is idnkit-2.
# The Initial Developer of the JPRS Revised Code is Japan Network
# Information Center ("JPNIC"), a Japanese association,
# Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda, Chiyoda-ku, Tokyo
# 101-0047, Japan.
# "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
# "Copyright (c) 2010-2013 Japan Registry Services Co., Ltd.  All rights reserved."
# Contributor(s): ______________________________________.
# 
# If you wish to allow use of your version of this file only under the
# above License(s) and not to allow others to use your version of this
# file, please indicate your decision by deleting the relevant provisions
# above and replacing them with the notice and other provisions required
# by the above License(s). If you do not delete the relevant provisions,
# a recipient may use your version of this file under either the above
# License(s).
#

package IDNKit::IDNA;

use 5.006;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);
use Carp;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration
# 	use IDNKit::IDNA ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	ENCODE_REGIST
	ENCODE_LOOKUP
	DECODE_REGIST
	DECODE_LOOKUP
	COMPARE_REGIST
	COMPARE_LOOKUP
	CHECK_REGIST
	CHECK_LOOKUP
	init
	set_local_check_file
	encode
	decode
	compare
	check
	get_idna_table_version
	get_libidn_version
	get_unicode_version
	get_version
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '2.3';

# Preloaded methods go here.
use constant ENCODE_REGIST  => -1;
use constant ENCODE_LOOKUP  => -2;
use constant DECODE_REGIST  => -3;
use constant DECODE_LOOKUP  => -4;
use constant COMPARE_REGIST => ENCODE_REGIST;
use constant COMPARE_LOOKUP => ENCODE_LOOKUP;
use constant CHECK_REGIST   => ENCODE_REGIST;
use constant CHECK_LOOKUP   => ENCODE_LOOKUP;

# Autoload methods go after =cut, and are processed by the autosplit program.

sub init {
    croak "IDNKit::IDNA::init() must not be called";
}

sub set_local_check_file {
    croak "IDNKit::IDNA::set_local_check_file() must not be called";
}

sub encode {
    croak "IDNKit::IDNA::encode() must not be called";
}

sub decode {
    croak "IDNKit::IDNA::decode() must not be called";
}

sub compare {
    croak "IDNKit::IDNA::compare() must not be called";
}

sub check {
    croak "IDNKit::IDNA::check() must not be called";
}

sub get_idna_table_version {
    croak "IDNKit::IDNA::get_idna_table_version() must not be called";
}

sub get_libidn_version {
    croak "IDNKit::IDNA::get_libidn_version() must not be called";
}

sub get_unicode_version {
    croak "IDNKit::IDNA::get_unicode_version() must not be called";
}

sub get_version {
    croak "IDNKit::IDNA::get_version() must not be called";
}

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

IDNKit::IDNA - Base class for IDNA conversion contexts.

=head1 DESCRIPTION

"IDNKit::IDNA" is a base class for IDNA conversion contexts.
"IDNKit::IDNA" itself is not designed to create conversion contexts.
Instead, use sub-classes of "IDNKit::IDNA", such as "IDNKit::IDNA2008".

=head1 CONSTANTS

The following constants are defined by the "IDNKit::IDNA" class:

=over 4

=item ENCODE_REGIST

Action value to encode a domain name with the domain name registration
protocol.
Also local-check is performed if the conversion context has
code point set.

=item ENCODE_LOOKUP

Action value to encode a domain name with the domain name lookup
protocol.
Also local-check is performed if the conversion context has
code point set.

=item DECODE_REGIST

Action value to decode a domain name with the domain name registration
protocol.
Also local-check is performed if the conversion context has
code point set.

=item DECODE_LOOKUP

Action value to decode a domain name with the domain name lookup
protocol.
Also local-check is performed if the conversion context has
code point set.

=item COMPARE_REGIST

Action value to compare two domain names with the domain name
registration protocol.
Also local-check is performed if the conversion context has
code point set.

=item COMPARE_LOOKUP

Action value to compare two domain names with the domain name
lookup protocol.
Also local-check is performed if the conversion context has
code point set.

=item CHECK_REGIST

Action value to validate a domain name with the domain name
registration protocol.
Also local-check is performed if the conversion context has
code point set.

=item CHECK_LOOKUP

Action value to validate a domain name with the domain name
lookup protocol.
Also local-check is performed if the conversion context has
code point set.

=back

=head1 METHODS

Subclasses of "IDNKit::IDNA" provide the following methods:

=over 4

=item init( )

=item init($file_name)

=item init([$line1, $line2, ...])

Initializes the conversion context instance.
If no argument is given (1st form), the context will have the default
settings upon return.
If a scalar argument $filename is given (2nd form), the context will
be initialized according with the configuration file specified by
$filename.
If $filename is empty, the context reads the configuration file of
the default path.
If a reference to an array is given (3rd form), the context will be
initialized according with the configuration strings.

=over 4

  $ctx->init("idn2.conf");

=back

is roughly equivalent to:

=over 4

  if (open(CONF, "idn2.conf")) {
      my @lines = <CONF>;
      close(CONF);
      $ctx->init(\@lines);
  }

=back

The method returns an "IDNKit::Result" instance which has the operation
result of this method.
If the operation has succeeded, error_code( ) method of
the "IDNKit::Result" instance returns "IDNKit::Result::IDN_SUCCESS".
Otherwise, it returns the corresponding error code (see "IDNKit::Result"
document for details).

=item set_local_check_file( )

=item set_local_check_file($filename)

Sets local code point set for local-check.
If $filename is given, the method reads a code point set from a file
$filename.
Otherwise it clears the local code point set the conversion context memories.

The method returns an "IDNKit::Result" instance which has the operation
result of this method.
If the operation has succeeded, error_code( ) method of
the "IDNKit::Result" instance returns "IDNKit::Result::IDN_SUCCESS".
Otherwise, it returns the corresponding error code (see "IDNKit::Result"
document for details).

=item encode($name)

=item encode($name, $actions)

Converts a domain name $name1 to ASCII.
The argument $actions specifies how $name is encoded.
If $actions is not given, the method performs DECODE_REGIST action
to encode $name.

The method returns an "IDNKit::Result" instance which has the operation
result of this method.
If the operation has succeeded, error_code( ) method of
the "IDNKit::Result" instance returns "IDNKit::Result::IDN_SUCCESS"
and name( ) method returns a converted domain name.
Otherwise, error_code( ) returns the corresponding error code
(see "IDNKit::Result" document for details), and name( ) returns undef.

=item decode($name)

=item decode($name, $actions)

Converts a domain name $name to UTF-8.
The argument $actions specifies how $name is decoded.
If $actions is not given, the method performs DECODE_REGIST action
to decode $name.

The method returns an "IDNKit::Result" instance which has the operation
result of this method.
If the operation has succeeded, error_code( ) method of
the "IDNKit::Result" instance returns "IDNKit::Result::IDN_SUCCESS"
and name( ) method returns a converted domain name.
Otherwise, error_code( ) returns the corresponding error code
(see "IDNKit::Result" document for details), and name( ) returns undef.

=item compare($name1, $name2)

=item compare($name1, $name2, $actions)

=item compare($name1, $name2, $actions1, $actions2)

Compares two domain names.
It encodes domain names $name1 and $name2 as the encode( ) method does,
and then compares them.
If only $name1 and $name2 are given (1st form), the method performs
COMPARE_LOOKUP action to encode $name1 and $name2.
If $actions is additionally given (2nd form), the method performs
$actions to encode both $name1 and $name2.
If $name1, $name2 and two action values $actions1 and $actions2 are
given (3rd form), the method performs $actions1 to encode $name1, and
$actions2 to encode $name2 respectively.

The method returns an "IDNKit::Result" instance which has the operation
result of this method.
If $name1 and $name2 are equivalent, error_code( ) method of
the "IDNKit::Result" instance returns "IDNKit::Result::IDN_SUCCESS".
Otherwise, it returns "IDNKit::Result::IDN_NEQ".
When an error occurs in the encoding processes, error_code( )
returns the corresponding error code (see "IDNKit::Result" document
for details).

=item check($name)

=item check($name, $actions)

Validates a domain name $name.
It tries to encode $name with an action value $actions.
If $actions is not given, it tries to encode $name with CHECK_REGIST
action.

The method returns an "IDNKit::Result" instance which has the operation
result of this method.
If the operation has succeeded, error_code( ) method of
the "IDNKit::Result" instance returns "IDNKit::Result::IDN_SUCCESS"
and name( ) method.
Otherwise, error_code( ) returns the corresponding error code
(see "IDNKit::Result" document for details).

=item get_unicode_version( )

Gets a version string of Unicode tables the context instance supports.

=item get_idna_table_version( )

Gets a version string of IDNA tables the context instance supports.

=item get_idna_libidn_version( )

Gets a version string of idnkit library.

=item get_version( )

Returns version information.
It returns a string which consists of a version of Unicode tables,
a version of IDNA tables and a version of idnkit library.

=back

=head1 SEE ALSO

L<IDNKit::IDNA2008>,
L<IDNKit::Result>

=head1 AUTHOR

Japan Registry Services Co., Ltd.(JPRS)

=cut
