#
#                           TERMS AND CONDITIONS
#                                   FOR
#                         OPEN SOURCE CODE LICENSE
#                               Version 1.1
# 
# Japan Registry Services Co., Ltd. ("JPRS"), a Japanese corporation
# having its head office at Chiyoda First Bldg. East 13F 3-8-1 Nishi-Kanda,
# Chiyoda-ku, Tokyo 101-0065, Japan, grants you the license for open source
# code specified in EXHIBIT A the "Code" subject to the following Terms and
# Conditions ("OSCL").
# 
# 1. License Grant.
#   JPRS hereby grants you a worldwide, royalty-free, non-exclusive
#   license, subject to third party intellectual property claims:
#   (a) under intellectual property rights (other than patent or
#       trademark) licensable by JPRS to use, reproduce, modify, display,
#       perform, sublicense and distribute the Code (or portions thereof)
#       with or without modifications, and/or as part of a derivative work;
#       or
#   (b) under claims of the infringement through the making, using,
#       offering to sell and/or otherwise disposing the JPRS Revised Code
#       (or portions thereof);
#   (c) the licenses granted in this Section 1(a) and (b) are effective on
#       the date JPRS first distributes the Code to you under the terms of
#       this OSCL;
#   (d) Notwithstanding the above stated terms, no patent license is
#       granted:
#       1)  for a code that you delete from the Code;
#       2)  separate from the Code; or
#       3)  for infringements caused by:
#            i) modification of the Code; or
#           ii) combination of the Code with other software or devices.
# 
# 2. Consents.
#   You agree that:
#   (a) you must include a copy of this OSCL and the notice set forth in
#       EXHIBIT A with every copy of the Code you distribute;
#   (b) you must include a copy of this OSCL and the notice set forth in
#       EXHIBIT A with every copy of binary form of the Code in the
#       documentation and/or other materials provided with the distribution;
#   (c) you may not offer or impose any terms on any source code version
#       that alters or restricts the applicable version of this OSCL or
#       the recipients' rights hereunder.
#   (d) If the terms and conditions are set forth in EXHIBIT A, you must
#       comply with those terms and conditions.
# 
# 3. Proprietary Information.
#   All trademarks, service marks, patents, copyrights, trade secrets, and
#   other proprietary rights in or related to the Code are and will remain
#   the exclusive property of JPRS or its licensors, whether or not
#   specifically recognized or perfected under local law except specified
#   in this OSCL; provided however you agree and understand that the JPRS
#   name may not be used to endorse or promote this Code without prior
#   written approval of JPRS.
# 
# 4. WARRANTY DISCLAIMER.
#   JPRS MAKES NO REPRESENTATIONS AND WARRANTIES REGARDING THE USE OF THE
#   CODE, NOR DOES JPRS MAKE ANY REPRESENTATIONS THAT THE CODE WILL BECOME
#   COMMERCIALLY AVAILABLE. JPRS, ITS AFFILIATES, AND ITS SUPPLIERS DO NOT
#   WARRANT OR REPRESENT THAT THE CODE IS FREE OF ERRORS OR THAT THE CODE
#   IS SUITABLE FOR TRANSLATION AND/OR LOCALIZATION. THE CODE IS PROVIDED
#   ON AN "AS IS" BASIS AND JPRS AND ITS SUPPLIERS HAVE NO OBLIGATION TO
#   CORRECT ERRORS OR TO SUPPORT THE CODE UNDER THIS OSCL FOR ANY REASON.
#   TO THE FULL EXTENT PERMITTED BY LAW, ALL OBLIGATIONS ARE HEREBY
#   EXCLUDED WHETHER EXPRESS, STATUTORY OR IMPLIED UNDER LAW, COURSE OF
#   DEALING, CUSTOM, TRADE USAGE, ORAL OR WRITTEN STATEMENT OR OTHERWISE,
#   INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY
#   OR FITNESS FOR A PARTICULAR PURPOSE CONCERNING THE CODE.
# 
# 5. NO LIABILITY.
#   UNDER NO CIRCUMSTANCES SHALL JPRS AND/OR ITS AFFILIATES, LICENSORS, OR
#   REPRESENTATIVES BE LIABLE FOR ANY DAMAGES INCLUDING BUT NOT LIMITED TO
#   CONSEQUENTIAL, INDIRECT, SPECIAL, PUNITIVE OR INCIDENTAL DAMAGES,
#   WHETHER FORESEEABLE OR UNFORESEEABLE, BASED ON YOUR CLAIMS, INCLUDING,
#   BUT NOT LIMITED TO, CLAIMS FOR LOSS OF DATA, GOODWILL, PROFITS, USE OF
#   MONEY, INTERRUPTION IN USE OR AVAILABILITY OF DATA, STOPPAGE, IMPLIED
#   WARRANTY, BREACH OF CONTRACT, MISREPRESENTATION, NEGLIGENCE, STRICT
#   LIABILITY IN TORT, OR OTHERWISE.
# 
# 6. Indemnification.
#   You hereby agree to indemnify, defend, and hold harmless JPRS for any
#   liability incurred by JRPS due to your terms of warranty, support,
#   indemnity, or liability offered by you to any third party.
# 
# 7. Termination.
# 7.1 This OSCL shall be automatically terminated in the events that:
#   (a) You fail to comply with the terms herein and fail to cure such
#       breach within 30 days of becoming aware of the breach;
#   (b) You initiate patent or copyright infringement litigation against
#       any party (including a cross-claim or counterclaim in a lawsuit)
#       alleging that the Code constitutes a direct or indirect patent or
#       copyright infringement, in such case, this OSCL to you shall
#       terminate as of the date such litigation is filed;
# 7.2 In the event of termination under Sections 7.1(a) or 7.1(b) above,
#     all end user license agreements (excluding distributors and
#     resellers) which have been validly granted by You or any distributor
#     hereunder prior to termination shall survive termination.
#
# 
# 8. General.
#   This OSCL shall be governed by, and construed and enforced in
#   accordance with, the laws of Japan. Any litigation or arbitration
#   between the parties shall be conducted exclusively in Tokyo, Japan
#   except written consent of JPRS provides other venue.
# 
# 
#                                EXHIBIT A
# 
# The original open source code of idnkit-2 is idnkit-1.0 developed and
# conceived by Japan Network Information Center ("JPNIC"), a Japanese
# association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
# Chiyoda-ku, Tokyo 101-0047, Japan, and JPRS modifies above original code
# under following Terms and Conditions set forth by JPNIC.
# 
#                                  JPNIC
# 
# Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved.
# 
# By using this file, you agree to the terms and conditions set forth bellow.
# 
#                       LICENSE TERMS AND CONDITIONS
# 
# The following License Terms and Conditions apply, unless a different
# license is obtained from Japan Network Information Center ("JPNIC"),
# a Japanese association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
# Chiyoda-ku, Tokyo 101-0047, Japan.
# 
# 1. Use, Modification and Redistribution (including distribution of any
#    modified or derived work) in source and/or binary forms is permitted
#    under this License Terms and Conditions.
# 
# 2. Redistribution of source code must retain the copyright notices as they
#    appear in each source code file, this License Terms and Conditions.
# 
# 3. Redistribution in binary form must reproduce the Copyright Notice,
#    this License Terms and Conditions, in the documentation and/or other
#    materials provided with the distribution. For the purposes of binary
#    distribution the "Copyright Notice" refers to the following language:
#    "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
# 
# 4. The name of JPNIC may not be used to endorse or promote products
#    derived from this Software without specific prior written approval of
#    JPNIC.
# 
# 5. Disclaimer/Limitation of Liability: THIS SOFTWARE IS PROVIDED BY JPNIC
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JPNIC BE LIABLE
#    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
#    BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
#    ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
# 
# 
#                        JPRS Public License Notice
#                                   For
#                                idnkit-2.
# 
# The contents of this file are subject to the Terms and Conditions for
# the Open Source Code License (the "OSCL"). You may not use this file
# except in compliance with above terms and conditions. A copy of the OSCL
# is available at <http://jprs.co.jp/idn/>.
# The JPRS Revised Code is idnkit-2.
# The Initial Developer of the JPRS Revised Code is Japan Network
# Information Center ("JPNIC"), a Japanese association,
# Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda, Chiyoda-ku, Tokyo
# 101-0047, Japan.
# "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
# "Copyright (c) 2010-2013 Japan Registry Services Co., Ltd.  All rights reserved."
# Contributor(s): ______________________________________.
# 
# If you wish to allow use of your version of this file only under the
# above License(s) and not to allow others to use your version of this
# file, please indicate your decision by deleting the relevant provisions
# above and replacing them with the notice and other provisions required
# by the above License(s). If you do not delete the relevant provisions,
# a recipient may use your version of this file under either the above
# License(s).
#

package IDNKit::IDNA2008;

use 5.006;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);
use Carp;
use IDNKit::IDNA;
use IDNKit::Result;

our @ISA = qw(IDNKit::IDNA);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration
# 	use IDNKit::IDNA2008 ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	MAP
	ASCLOWER
	RTCONV 
	PROHCHECK
	UNASCHECK
	NFCCHECK
	PREFCHECK
	HYPHCHECK
	COMBCHECK
	CTXJCHECK
	CTXOCHECK
	CTXOLITECHECK
	BIDICHECK
	LOCALCHECK
	IDNCONV
	LENCHECK
	RTCHECK
	UNDOIFERR
	new
	init
	set_local_check_file
	encode
	decode
	compare
	check
	get_idna_table_version
	get_libidn_version
	get_unicode_version
	get_version
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '2.3';

require XSLoader;
XSLoader::load('IDNKit::IDNA2008', $VERSION);

# Preloaded methods go here.
use constant MAP           => 0x00000002;
use constant ASCLOWER      => 0x00000004;
use constant RTCONV        => 0x00000008;
use constant PROHCHECK     => 0x00000010;
use constant UNASCHECK     => 0x00000020;
use constant NFCCHECK      => 0x00000040;
use constant PREFCHECK     => 0x00000080;
use constant HYPHCHECK     => 0x00000100;
use constant COMBCHECK     => 0x00000200;
use constant CTXJCHECK     => 0x00000400;
use constant CTXOCHECK     => 0x00000800;
use constant CTXOLITECHECK => 0x00001000;
use constant BIDICHECK     => 0x00002000;
use constant LOCALCHECK    => 0x00004000;
use constant IDNCONV       => 0x00008000;
use constant LENCHECK      => 0x00010000;
use constant RTCHECK       => 0x00020000;
use constant UNDOIFERR     => 0x40000000;

sub adjust_action_value {
    my ($actions) = @_;

    if ($actions == IDNKit::IDNA::ENCODE_REGIST
	|| $actions == IDNKit::IDNA::COMPARE_REGIST
	|| $actions == IDNKit::IDNA::CHECK_REGIST) {
	return MAP | ASCLOWER | RTCONV | PROHCHECK | UNASCHECK | NFCCHECK
	    | PREFCHECK | HYPHCHECK | COMBCHECK | CTXJCHECK | CTXOCHECK
	    | BIDICHECK | IDNCONV | LENCHECK | RTCHECK | LOCALCHECK;
    } 
    if ($actions == IDNKit::IDNA::ENCODE_LOOKUP
	|| $actions == IDNKit::IDNA::COMPARE_LOOKUP
	|| $actions == IDNKit::IDNA::CHECK_LOOKUP) {
	return MAP | ASCLOWER | RTCONV | PROHCHECK | UNASCHECK | NFCCHECK
	    | PREFCHECK | COMBCHECK | CTXJCHECK | CTXOLITECHECK 
	    | BIDICHECK | IDNCONV | LENCHECK | RTCHECK | LOCALCHECK;
    }
    if ($actions == IDNKit::IDNA::DECODE_REGIST) {
	return MAP | ASCLOWER | IDNCONV | PROHCHECK | UNASCHECK | NFCCHECK
	    | PREFCHECK | HYPHCHECK | COMBCHECK | CTXJCHECK | CTXOCHECK 
	    | BIDICHECK | RTCHECK | LOCALCHECK;
    }
    if ($actions == IDNKit::IDNA::DECODE_LOOKUP) {
	return MAP | ASCLOWER | IDNCONV | PROHCHECK | UNASCHECK | NFCCHECK
	    | PREFCHECK | COMBCHECK | CTXJCHECK | CTXOLITECHECK 
	    | BIDICHECK | RTCHECK | LOCALCHECK;
    }

    return $actions;
}

sub init {
    my ($self) = shift;
    my $result_raw;

    if (@_ == 0 || !defined($_[0])) {
	$result_raw = setdefaults_raw($self);
    } elsif (ref($_[0]) eq '') {
	$result_raw = loadfile_raw($self, $_[0]);
    } elsif (ref($_[0]) eq 'ARRAY') {
	$result_raw = loadstrings_raw($self, $_[0]);
    } else {
	croak "IDNKit::IDNA2008::init() - invalid argument";
    }

    return IDNKit::Result->new($result_raw);
}

sub set_local_check_file {
    my ($self) = shift;
    my $filename = undef;
    my $result_raw;

    $filename = $_[0] if (@_ > 0 && defined($_[0]));

    if (!defined($filename)) {
	$result_raw = clear_local_check_file_raw($self);
    } else {
	$result_raw = set_local_check_file_raw($self, $filename);
    }
    return IDNKit::Result->new($result_raw);
}

sub encode {
    my ($self) = shift;
    my $name = '';
    my $actions = IDNKit::IDNA::ENCODE_REGIST;

    if (@_ <= 0 || !defined($_[0])) {
	croak "IDNKit::IDNA2008::encode() - invalid argument";
    }

    $name    = $_[0];
    $actions = $_[1] if (@_ > 1 && defined($_[1]));

    my $result_raw = encode_raw($self, $name, adjust_action_value($actions));
    return IDNKit::Result->new($result_raw->[0], $result_raw->[1]);
}

sub decode {
    my ($self) = shift;
    my $name = '';
    my $actions = IDNKit::IDNA::DECODE_REGIST;

    if (@_ <= 0 || !defined($_[0])) {
	croak "IDNKit::IDNA2008::decode() - invalid argument";
    }

    $name    = $_[0];
    $actions = $_[1] if (@_ > 1 && defined($_[1]));

    my $result_raw = decode_raw($self, $name, adjust_action_value($actions));
    return IDNKit::Result->new($result_raw->[0], $result_raw->[1]);
}

sub compare {
    my ($self) = shift;
    my $name1 = '';
    my $name2 = '';
    my $actions1 = IDNKit::IDNA::COMPARE_LOOKUP;
    my $actions2 = IDNKit::IDNA::COMPARE_LOOKUP;

    if (@_ <= 1 || !defined($_[0]) || !defined($_[1])) {
	croak "IDNKit::IDNA2008::compare() - invalid argument";
    }

    $name1    = $_[0];
    $name2    = $_[1];
    $actions1 = $_[2] if (@_ > 2 && defined($_[2]));
    $actions2 = (@_ > 3 && defined($_[2])) ? $_[3] : $actions1;

    my $result_raw = compare_raw($self, $name1, $name2,
				 adjust_action_value($actions1),
				 adjust_action_value($actions2));
    return IDNKit::Result->new($result_raw);
}

sub check {
    my ($self) = shift;
    my $name = '';
    my $actions = IDNKit::IDNA::CHECK_REGIST;

    if (@_ <= 0 || !defined($_[0])) {
	croak "IDNKit::IDNA2008::check() - invalid argument";
    }

    $name    = $_[0] if (@_ > 0 && defined($_[0]));
    $actions = $_[1] if (@_ > 1 && defined($_[1]));

    my $result_raw = check_raw($self, $name, adjust_action_value($actions));
    return IDNKit::Result->new($result_raw);
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

IDNKit::IDNA2008 - IDNA2008 conversion context.

=head1 SYNOPSIS

  use IDNKit::IDNAFactory;

  $ctx = IDNKit::IDNAFactory::create(IDNKit::IDNAFactory::IDNA2008);
  $result = $ctx->init("idn2.conf");
  if ($result->error_code() != IDNKit::Result::SUCCESS) {
      die "IDNKit::IDNAFactory::init() failed\n";
  }

  $result = $ctx->decode("xn--h1ahn.jp");
  if ($result->error_code() != IDNKit::Result::SUCCESS) {
      die "IDNKit::IDNAFactory::decode() failed\n";
  }

  print $result->name();

=head1 DESCRIPTION

An instance of "IDNKit::IDNA2008" class represents IDNA2008 version of
an IDNA conversion context.

Its main functions are encoding (conversion to ASCII) and decoding
(conversion to UTF-8) domain names, according with RFC5894; 
"Internationalized Domain Names in Applications (IDNA): Protocol".

Note that "IDNKit::IDNA2008" class doesn't provide a constructor.
To create an "IDNKit::IDNA2008" instance, invoke create( ) class method
of "IDNKit::IDNAFactory" class (see SYNOPSIS).

=head1 CONSTANTS

The following constants are defined by the "IDNKit::IDNA2008" class:

=over 4

=item MAP Z<>

Atomic MAP action.
It performs mappings (NFC, Lowercase conversion, etc.).

=item ASCLOWER

Atomic ASCLOWER action.
It converts ASCII uppercase letters (A..Z) to lowercase (a..z).

=item RTCONV 

Atomic RTCONV action.
It converts a domain name from Punycode to UTF-8.

=item PROHCHECK

Atomic PROHCHECK action.
It checks prohibited code points.

=item UNASCHECK

Atomic UNASCHECK action.
It checks unassigned code points.

=item NFCCHECK

Atomic NFCCHECK action.
It checks labels are in NFC.

=item PREFCHECK

Atomic PREFCHECK action.
It checks labels containing "--" in the 3rd and 4th characters.

=item HYPHCHECK

Atomic HYPHCHECK action.
It checks labels beginning/ending with "-".

=item COMBCHECK

Atomic COMBCHECK action.
It checks labels beginning with a combining mark.

=item CTXJCHECK

Atomic CTXJCHECK action.
It checks CONTEXTJ code points.

=item CTXOCHECK

Atomic CTXOCHECK action.
It checks CONTEXTO code points for the registration protocol.

=item CTXOLITECHECK

Atomic CTXOLITECHECK action.
It checks CONTEXTO code points for the lookup protocol.

=item BIDICHECK

Atomic BIDICHECK action.
It checks requirements specified in [IDNA2008-BIDI].

=item LOCALCHECK

Atomic LOCALCHECK action.
Perform local check.

=item IDNCONV

Atomic IDNCONV action.
Convert labels in UTF-8 to Punycode.

=item LENCHECK

Atomic LENCHECK action.
It checks length of each label.

=item RTCHECK

Atomic RTCHECK action.
Perform round trip check for each label.

=item UNDOIFERR

UNDOIFERR special action.
If any step fails, the original input name is returned.

=back

=head1 METHODS

The following methods are provided by the "IDNKit::IDNA2008":

=over 4

=item init( )

=item init($file_name)

=item init([$line1, $line2, ...])

=item set_local_check_file( )

=item set_local_check_file($filename)

=item encode($name)

=item encode($name, $actions)

=item decode($name)

=item decode($name, $actions)

=item compare($name1, $name2)

=item compare($name1, $name2, $actions)

=item compare($name1, $name2, $actions1, $actions2)

=item check($name)

=item check($name, $actions)

=item get_unicode_version( )

=item get_idna_table_version( )

=item get_idna_libidn_version( )

=item get_version( )

These methods are inherited from "IDNKit::IDNA".

=back

=head1 SEE ALSO

L<IDNKit::IDNAFactory>,
L<IDNKit::IDNA>,
L<IDNKit::Result>

=head1 AUTHOR

Japan Registry Services Co., Ltd.(JPRS)

=cut
