#
#                           TERMS AND CONDITIONS
#                                   FOR
#                         OPEN SOURCE CODE LICENSE
#                               Version 1.1
# 
# Japan Registry Services Co., Ltd. ("JPRS"), a Japanese corporation
# having its head office at Chiyoda First Bldg. East 13F 3-8-1 Nishi-Kanda,
# Chiyoda-ku, Tokyo 101-0065, Japan, grants you the license for open source
# code specified in EXHIBIT A the "Code" subject to the following Terms and
# Conditions ("OSCL").
# 
# 1. License Grant.
#   JPRS hereby grants you a worldwide, royalty-free, non-exclusive
#   license, subject to third party intellectual property claims:
#   (a) under intellectual property rights (other than patent or
#       trademark) licensable by JPRS to use, reproduce, modify, display,
#       perform, sublicense and distribute the Code (or portions thereof)
#       with or without modifications, and/or as part of a derivative work;
#       or
#   (b) under claims of the infringement through the making, using,
#       offering to sell and/or otherwise disposing the JPRS Revised Code
#       (or portions thereof);
#   (c) the licenses granted in this Section 1(a) and (b) are effective on
#       the date JPRS first distributes the Code to you under the terms of
#       this OSCL;
#   (d) Notwithstanding the above stated terms, no patent license is
#       granted:
#       1)  for a code that you delete from the Code;
#       2)  separate from the Code; or
#       3)  for infringements caused by:
#            i) modification of the Code; or
#           ii) combination of the Code with other software or devices.
# 
# 2. Consents.
#   You agree that:
#   (a) you must include a copy of this OSCL and the notice set forth in
#       EXHIBIT A with every copy of the Code you distribute;
#   (b) you must include a copy of this OSCL and the notice set forth in
#       EXHIBIT A with every copy of binary form of the Code in the
#       documentation and/or other materials provided with the distribution;
#   (c) you may not offer or impose any terms on any source code version
#       that alters or restricts the applicable version of this OSCL or
#       the recipients' rights hereunder.
#   (d) If the terms and conditions are set forth in EXHIBIT A, you must
#       comply with those terms and conditions.
# 
# 3. Proprietary Information.
#   All trademarks, service marks, patents, copyrights, trade secrets, and
#   other proprietary rights in or related to the Code are and will remain
#   the exclusive property of JPRS or its licensors, whether or not
#   specifically recognized or perfected under local law except specified
#   in this OSCL; provided however you agree and understand that the JPRS
#   name may not be used to endorse or promote this Code without prior
#   written approval of JPRS.
# 
# 4. WARRANTY DISCLAIMER.
#   JPRS MAKES NO REPRESENTATIONS AND WARRANTIES REGARDING THE USE OF THE
#   CODE, NOR DOES JPRS MAKE ANY REPRESENTATIONS THAT THE CODE WILL BECOME
#   COMMERCIALLY AVAILABLE. JPRS, ITS AFFILIATES, AND ITS SUPPLIERS DO NOT
#   WARRANT OR REPRESENT THAT THE CODE IS FREE OF ERRORS OR THAT THE CODE
#   IS SUITABLE FOR TRANSLATION AND/OR LOCALIZATION. THE CODE IS PROVIDED
#   ON AN "AS IS" BASIS AND JPRS AND ITS SUPPLIERS HAVE NO OBLIGATION TO
#   CORRECT ERRORS OR TO SUPPORT THE CODE UNDER THIS OSCL FOR ANY REASON.
#   TO THE FULL EXTENT PERMITTED BY LAW, ALL OBLIGATIONS ARE HEREBY
#   EXCLUDED WHETHER EXPRESS, STATUTORY OR IMPLIED UNDER LAW, COURSE OF
#   DEALING, CUSTOM, TRADE USAGE, ORAL OR WRITTEN STATEMENT OR OTHERWISE,
#   INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY
#   OR FITNESS FOR A PARTICULAR PURPOSE CONCERNING THE CODE.
# 
# 5. NO LIABILITY.
#   UNDER NO CIRCUMSTANCES SHALL JPRS AND/OR ITS AFFILIATES, LICENSORS, OR
#   REPRESENTATIVES BE LIABLE FOR ANY DAMAGES INCLUDING BUT NOT LIMITED TO
#   CONSEQUENTIAL, INDIRECT, SPECIAL, PUNITIVE OR INCIDENTAL DAMAGES,
#   WHETHER FORESEEABLE OR UNFORESEEABLE, BASED ON YOUR CLAIMS, INCLUDING,
#   BUT NOT LIMITED TO, CLAIMS FOR LOSS OF DATA, GOODWILL, PROFITS, USE OF
#   MONEY, INTERRUPTION IN USE OR AVAILABILITY OF DATA, STOPPAGE, IMPLIED
#   WARRANTY, BREACH OF CONTRACT, MISREPRESENTATION, NEGLIGENCE, STRICT
#   LIABILITY IN TORT, OR OTHERWISE.
# 
# 6. Indemnification.
#   You hereby agree to indemnify, defend, and hold harmless JPRS for any
#   liability incurred by JRPS due to your terms of warranty, support,
#   indemnity, or liability offered by you to any third party.
# 
# 7. Termination.
# 7.1 This OSCL shall be automatically terminated in the events that:
#   (a) You fail to comply with the terms herein and fail to cure such
#       breach within 30 days of becoming aware of the breach;
#   (b) You initiate patent or copyright infringement litigation against
#       any party (including a cross-claim or counterclaim in a lawsuit)
#       alleging that the Code constitutes a direct or indirect patent or
#       copyright infringement, in such case, this OSCL to you shall
#       terminate as of the date such litigation is filed;
# 7.2 In the event of termination under Sections 7.1(a) or 7.1(b) above,
#     all end user license agreements (excluding distributors and
#     resellers) which have been validly granted by You or any distributor
#     hereunder prior to termination shall survive termination.
#
# 
# 8. General.
#   This OSCL shall be governed by, and construed and enforced in
#   accordance with, the laws of Japan. Any litigation or arbitration
#   between the parties shall be conducted exclusively in Tokyo, Japan
#   except written consent of JPRS provides other venue.
# 
# 
#                                EXHIBIT A
# 
# The original open source code of idnkit-2 is idnkit-1.0 developed and
# conceived by Japan Network Information Center ("JPNIC"), a Japanese
# association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
# Chiyoda-ku, Tokyo 101-0047, Japan, and JPRS modifies above original code
# under following Terms and Conditions set forth by JPNIC.
# 
#                                  JPNIC
# 
# Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved.
# 
# By using this file, you agree to the terms and conditions set forth bellow.
# 
#                       LICENSE TERMS AND CONDITIONS
# 
# The following License Terms and Conditions apply, unless a different
# license is obtained from Japan Network Information Center ("JPNIC"),
# a Japanese association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
# Chiyoda-ku, Tokyo 101-0047, Japan.
# 
# 1. Use, Modification and Redistribution (including distribution of any
#    modified or derived work) in source and/or binary forms is permitted
#    under this License Terms and Conditions.
# 
# 2. Redistribution of source code must retain the copyright notices as they
#    appear in each source code file, this License Terms and Conditions.
# 
# 3. Redistribution in binary form must reproduce the Copyright Notice,
#    this License Terms and Conditions, in the documentation and/or other
#    materials provided with the distribution. For the purposes of binary
#    distribution the "Copyright Notice" refers to the following language:
#    "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
# 
# 4. The name of JPNIC may not be used to endorse or promote products
#    derived from this Software without specific prior written approval of
#    JPNIC.
# 
# 5. Disclaimer/Limitation of Liability: THIS SOFTWARE IS PROVIDED BY JPNIC
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JPNIC BE LIABLE
#    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
#    BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
#    ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
# 
# 
#                        JPRS Public License Notice
#                                   For
#                                idnkit-2.
# 
# The contents of this file are subject to the Terms and Conditions for
# the Open Source Code License (the "OSCL"). You may not use this file
# except in compliance with above terms and conditions. A copy of the OSCL
# is available at <http://jprs.co.jp/idn/>.
# The JPRS Revised Code is idnkit-2.
# The Initial Developer of the JPRS Revised Code is Japan Network
# Information Center ("JPNIC"), a Japanese association,
# Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda, Chiyoda-ku, Tokyo
# 101-0047, Japan.
# "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
# "Copyright (c) 2010-2012 Japan Registry Services Co., Ltd.  All rights reserved."
# Contributor(s): ______________________________________.
# 
# If you wish to allow use of your version of this file only under the
# above License(s) and not to allow others to use your version of this
# file, please indicate your decision by deleting the relevant provisions
# above and replacing them with the notice and other provisions required
# by the above License(s). If you do not delete the relevant provisions,
# a recipient may use your version of this file under either the above
# License(s).
#

package IDNKit::Result;

use 5.006;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);
use Carp;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration
# 	use IDNKit::Result ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	SUCCESS
	NOT_FOUND
	INVALID_ENCODING
	INVALID_SYNTAX
	INVALID_NAME
	INVALID_MESSAGE
	INVALID_ACTION
	INVALID_CODE_POINT
	BUFFER_OVERFLOW
	NO_ENTRY
	NO_MEMORY
	NO_FILE
	NO_MAPPING
	CONTEXT_REQUIRED
	PROHCHECK_ERROR
	UNASCHECK_ERROR
	NFCCHECK_ERROR
	PREFCHECK_ERROR
	HYPHCHECK_ERROR
	COMBCHECK_ERROR
	CTXJCHECK_ERROR
	CTXOCHECK_ERROR
	BIDICHECK_ERROR
	LOCALCHECK_ERROR
	LENCHECK_ERROR
	RTCHECK_ERROR
	TR46CHECK_ERROR
	NEQ
	FAILURE
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '2.3';

# Preloaded methods go here.
use constant SUCCESS            =>  0;
use constant NOT_FOUND          =>  1;
use constant INVALID_ENCODING   =>  2;
use constant INVALID_SYNTAX     =>  3;
use constant INVALID_NAME       =>  4;
use constant INVALID_MESSAGE    =>  5;
use constant INVALID_ACTION     =>  6;
use constant INVALID_CODE_POINT =>  7;
use constant BUFFER_OVERFLOW    =>  8;
use constant NO_ENTRY           =>  9;
use constant NO_MEMORY          => 10;
use constant NO_FILE            => 11;
use constant NO_MAPPING         => 12;
use constant CONTEXT_REQUIRED   => 13;
use constant PROHCHECK_ERROR    => 14;
use constant UNASCHECK_ERROR    => 15;
use constant NFCCHECK_ERROR     => 16;
use constant PREFCHECK_ERROR    => 17;
use constant HYPHCHECK_ERROR    => 18;
use constant COMBCHECK_ERROR    => 19;
use constant CTXJCHECK_ERROR    => 20;
use constant CTXOCHECK_ERROR    => 21;
use constant BIDICHECK_ERROR    => 22;
use constant LOCALCHECK_ERROR   => 23;
use constant LENCHECK_ERROR     => 24;
use constant RTCHECK_ERROR      => 25;
use constant TR46CHECK_ERROR    => 26;
use constant NEQ                => 27;
use constant FAILURE            => 28;

my @messages = (#  0 -  4
		"success",
		"not found",
		"invalid encoding found",
		"syntax error",
		"invalid name",
	     
		#  5 -  9
		"invalid message",
		"invalid action",
		"invalid code point",
		"buffer overflow",
		"no such entry",

		# 10 - 14
		"out of memory",
		"no such file",
		"no mapping to output codeset",
		"context information required",
		"prohibited code point check failed",

		# 15 - 19
		"unassigned code point check failed",
		"NFC validation check failed",
		"ACE prefix check failed",
		"hyphen check failed",
		"combining character check failed",

		# 20 - 24
		"CONTEXTJ check failed",
		"CONTEXTO check failed",
		"Bidi check failed",
		"local check failed",
		"label length check failed",

		# 25 - 29
		"round trip check failed",
		"TR #46 check failed",
		"not equivalent domain names",
		"generic failure");

my $unknown_code_message = "unknown result code";

# Autoload methods go after =cut, and are processed by the autosplit program.
sub new {
    my ($type, $error_code, $name) = @_;

    my $self = {'name'       => $name,
		'error_code' => $error_code};

    return bless($self, $type);
}

sub error_code {
    my ($self) = shift;
    return $self->{error_code};
}

sub message {
    my ($self) = shift;

    my $code = $self->{error_code};
    if (!defined($code) || $code < 0 || $code >= @messages) {
	return $unknown_code_message;
    }
    return $messages[$code];
}

sub name {
    my ($self) = shift;
    return $self->{name};
}

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

IDNKit::Result - Operation result of IDNA conversion context.

=head1 SYNOPSIS

  use IDNKit::IDNAFactory;

  $ctx = IDNKit::IDNAFactory::create();
  $result = $ctx->decode("xn--h1ahn.jp");
  if ($result->error_code() != IDNKit::Result::SUCCESS) {
      die "IDNKit::IDNAFactory::decode() failed\n";
  }

  print $result->name();

=head1 DESCRIPTION

An "IDNKit::Result" instance stores an operation result of an IDNA
conversion context.

Some methods of IDNA conversion context classes, such as "IDNKit::IDNA2008"
class, return an "IDNKit::Result" instance.

=head1 CONSTRUCTOR

=over 4

=item new($error_code)

=item new($error_code, $name)

Creates an "IDNKit::Result" instance which has an error code $error_code
as operation result.
The instance optionally has a domain name $name as conversion result
of encode( ) or decode( ) operation.

=back

=head1 METHODS

=over 4

=item error_code()

Returns an error code stored in the instance.

=item message()

Returns an error message corresponding with an error code stored
in the instance.

=item name()

Returns a string of conversion result.
Use this method to retrieve a domain name converted by encode( ) or
decode( ) method of IDNA conversion context class.

=back

=head1 SEE ALSO

L<IDNKit::IDNA>,
L<IDNKit::IDNA2008>

=head1 AUTHOR

Japan Registry Services Co., Ltd.(JPRS)

=cut
